@@ -1,4 +1,4 @@
-# Copyright 2008-2019 Kouhei Sutou <kou@cozmixng.org>
+# Copyright 2008-2020  Sutou Kouhei <kou@cozmixng.org>
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -15,15 +15,14 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
 begin
-  require "pkg-config/version"
+  require_relative "pkg-config/version"
 rescue LoadError
 end
 
+require "English"
+require "pathname"
 require "rbconfig"
-
-require 'shellwords'
-require 'English'
-require 'pathname'
+require "shellwords"
 
 class PackageConfig
   SEPARATOR = File::PATH_SEPARATOR
@@ -249,13 +248,15 @@ class PackageConfig
     @path_position
   end
 
-  def collect_requires(&block)
+  def collect_requires(processed_packages={}, &block)
     packages = []
     targets = yield(self)
     targets.each do |name|
+      next if processed_packages.key?(name)
       package = self.class.new(name, @options)
+      processed_packages[name] = package
       packages << package
-      packages.concat(package.collect_requires(&block))
+      packages.concat(package.collect_requires(processed_packages, &block))
     end
     packages_without_self = packages.reject do |package|
       package.name == @name
@@ -358,7 +359,7 @@ class PackageConfig
     all_flags = {}
     flags = []
     in_option = false
-    libs_command_line.gsub(/-([Ll]) /, '\1').split.each do |arg|
+    libs_command_line.gsub(/-([Ll]) /, "\\1").split.each do |arg|
       if in_option
         flags << arg
         in_option = false
@@ -384,7 +385,7 @@ class PackageConfig
     @declarations = {}
     File.open(pc_path) do |input|
       input.each_line do |line|
-        line = line.gsub(/#.*/, '').strip
+        line = line.gsub(/#.*/, "").strip
         next if line.empty?
         case line
         when /^(#{IDENTIFIER_RE})=/
@@ -398,7 +399,7 @@ class PackageConfig
 
   def parse_requires(requires)
     return [] if requires.nil?
-    requires_without_version = requires.gsub(/[<>]?=\s*[\d.a-zA-Z_-]+\s*/, '')
+    requires_without_version = requires.gsub(/[<>]?=\s*[\d.a-zA-Z_-]+\s*/, "")
     requires_without_version.split(/[,\s]+/)
   end
 
@@ -438,6 +439,11 @@ class PackageConfig
       "/opt/X11/lib/pkgconfig",
       "/usr/share/pkgconfig",
     ]
+    if Object.const_defined?(:RubyInstaller)
+      mingw_bin_path = RubyInstaller::Runtime.msys2_installation.mingw_bin_path
+      mingw_pkgconfig_path = Pathname.new(mingw_bin_path) + "../lib/pkgconfig"
+      default_paths.unshift(mingw_pkgconfig_path.cleanpath.to_s)
+    end
     libdir = ENV["PKG_CONFIG_LIBDIR"]
     default_paths.unshift(libdir) if libdir
 
@@ -514,7 +520,7 @@ module PKGConfig
   end
 
   def msvc?
-    /mswin/.match(RUBY_PLATFORM) and /^cl\b/.match(RbConfig::CONFIG['CC'])
+    /mswin/.match(RUBY_PLATFORM) and /^cl\b/.match(RbConfig::CONFIG["CC"])
   end
 
   def package_config(package)
@@ -589,18 +595,18 @@ module PKGConfig
       dldflags = libs(pkg)
       dldflags = (Shellwords.shellwords(dldflags) -
                   Shellwords.shellwords(libraries))
-      dldflags = dldflags.map {|s| /\s/ =~ s ? "\"#{s}\"" : s }.join(' ')
-      $libs   += ' ' + libraries
+      dldflags = dldflags.map {|s| /\s/ =~ s ? "\"#{s}\"" : s }.join(" ")
+      $libs   += " " + libraries
       if /mswin/ =~ RUBY_PLATFORM
-        $DLDFLAGS += ' ' + dldflags
+        $DLDFLAGS += " " + dldflags
       else
-        $LDFLAGS += ' ' + dldflags
+        $LDFLAGS += " " + dldflags
       end
-      $CFLAGS += ' ' + cflags_only_other(pkg)
+      $CFLAGS += " " + cflags_only_other(pkg)
       if defined?($CXXFLAGS)
-        $CXXFLAGS += ' ' + cflags_only_other(pkg)
+        $CXXFLAGS += " " + cflags_only_other(pkg)
       end
-      $INCFLAGS += ' ' + cflags_only_I(pkg)
+      $INCFLAGS += " " + cflags_only_I(pkg)
     end
     enough_version
   end
